import React from 'react';
import {Motion, spring} from 'react-motion';
import {shouldComponentUpdate} from 'react/lib/ReactComponentWithPureRenderMixin';


const SPRING_PRECISION = 1;


const WAITING = 'WAITING';
const RESIZING = 'RESIZING';
const RESTING = 'RESTING';
const IDLING = 'IDLING';


const noop = () => null;
const css = {
  collapse: 'ReactCollapse--collapse',
  content: 'ReactCollapse--content'
};


export const Collapse = React.createClass({
  propTypes: {
    isOpened: React.PropTypes.bool.isRequired,
    springConfig: React.PropTypes.objectOf(React.PropTypes.number),

    theme: React.PropTypes.objectOf(React.PropTypes.string),
    style: React.PropTypes.object,

    onRender: React.PropTypes.func,
    onRest: React.PropTypes.func,
    onMeasure: React.PropTypes.func,

    children: React.PropTypes.node.isRequired
  },


  getDefaultProps() {
    return {
      style: {},
      theme: css,
      onRender: noop,
      onRest: noop,
      onMeasure: noop
    };
  },


  getInitialState() {
    return {currentState: IDLING, from: 0, to: 0};
  },


  componentDidMount() {
    const {isOpened, onRest} = this.props;
    if (isOpened) {
      const to = this.content.clientHeight;
      this.setState({currentState: IDLING, from: to, to});
    }
    onRest();
  },


  componentWillReceiveProps(nextProps) {
    if (this.state.currentState === IDLING && (nextProps.isOpened || this.props.isOpened)) {
      this.setState({currentState: WAITING});
    }
  },


  shouldComponentUpdate,


  componentDidUpdate(_, prevState) {
    const {isOpened, onRest, onMeasure} = this.props;

    if (this.state.currentState === IDLING) {
      onRest();
      return;
    }

    if (prevState.to !== this.state.to) {
      onMeasure({height: this.state.to, width: this.content.clientWidth});
    }

    const from = this.wrapper.clientHeight;
    const to = isOpened ? this.content.clientHeight : 0;

    if (from !== to) {
      this.setState({currentState: RESIZING, from, to});
      return;
    }

    if (this.state.currentState === RESTING) {
      this.setState({currentState: IDLING, from, to});
    }
  },


  componentWillUnmount() {
    cancelAnimationFrame(this.raf);
  },


  onContentRef(content) {
    this.content = content;
  },


  onWrapperRef(wrapper) {
    this.wrapper = wrapper;
  },


  onRest() {
    this.raf = requestAnimationFrame(this.setResting);
  },


  setResting() {
    this.setState({currentState: RESTING});
  },


  isHeightAuto() {
    return this.state.currentState === IDLING && this.state.to;
  },


  getExtraStyles() {
    if (this.isHeightAuto()) {
      return {height: 'auto'};
    }

    const extraStyle = {overflow: 'hidden'};
    if (this.state.currentState === WAITING && !this.state.to) {
      return {...extraStyle, height: 0};
    }

    return extraStyle;
  },


  getMotionProps() {
    const {springConfig} = this.props;

    return this.state.currentState === IDLING ? {
      // When completely stable, instantly jump to the position
      defaultStyle: {height: this.state.to},
      style: {height: this.state.to}
    } : {
      // Otherwise, animate
      defaultStyle: {height: this.state.from},
      style: {height: spring(this.state.to, {precision: SPRING_PRECISION, ...springConfig})}
    };
  },


  renderContent({height}) {
    const {
      isOpened: _isOpened,
      springConfig: _springConfig,
      theme,
      style,
      onRender,
      onRest: _onRest,
      onMeasure: _onMeasure,
      children,
      ...props
    } = this.props;

    const {
      from,
      to
    } = this.state;

    // DANGEROUS, use with caution, never do setState with it
    onRender({current: height, from, to});

    return (
      <div
        ref={this.onWrapperRef}
        className={theme.collapse}
        style={{
          ...this.getExtraStyles(),
          ...(this.isHeightAuto() ? {} : {height: Math.max(0, height)}),
          ...style
        }}
        {...props}>
        <div ref={this.onContentRef} className={theme.content}>{children}</div>
      </div>
    );
  },


  render() {
    return (
      <Motion
        {...this.getMotionProps()}
        onRest={this.onRest}
        children={this.renderContent} />
    );
  }
});
